from .fileInteraction import (
    getFilePath,
    writeFile,
    createDirectory,
    readData,
    asyncListDir,
    asyncReadJSON,
)
from .userInteraction import (
    askYNQuestion,
    getInputFilePath,
    printArray,
    selectOptionQuestion,
    signature,
)
from .processCSVInChunks import processCSVInChunks



from .askYNQuestion import askYNQuestion
from .getInputFilePath import getInputFilePath
from .printArray import printArray
from .selectOptionQuestion import selectOptionQuestion
from .signature import signature



from alive_progress import alive_bar
import pandas as pd

# Process any CSV file in chunks instead of whole file at once
async def processCSVInChunks(csv_file, chunk_size, process_function, pf_params):
    # Create a generator to read the CSV file in chunks
    chunk_reader = pd.read_csv(csv_file, chunksize=chunk_size)

    # Process each chunk and concatenate the results
    processed_chunks = []
    total_chunks = int(sum(1 for row in open(csv_file, "r")) / chunk_size)
    with alive_bar(total_chunks, force_tty=True, bar="filling", spinner="waves") as bar:
        bar.title = "Processing file -> " + csv_file
        for chunk in chunk_reader:
            processed_chunk = process_function(chunk, pf_params)
            processed_chunks.append(processed_chunk)
            bar()
    # Concatenate the processed chunks into a single DataFrame
    df = pd.concat(processed_chunks, ignore_index=True)

    return df


import pandas
import warnings


async def readData(file_path) -> pandas.DataFrame:
    try:
        warnings.filterwarnings("error", category=pandas.errors.DtypeWarning)
        # Get the data from file
        # TODO convert to stream read
        if file_path.endswith(".csv"):
            read_data = pandas.read_csv(f"""{file_path}""")
        elif file_path.endswith(".xlsx"):
            read_data = pandas.read_excel(f"""{file_path}""")
        elif file_path.endswith(".json"):
            read_data = pandas.read_json(f"""{file_path}""")
        warnings.resetwarnings()
    except pandas.errors.DtypeWarning as pe:
        raise Exception(
            "Error encountered while reading the file: " + pe.args[0]
        ) from pe
    except Exception as exc:
        raise Exception("Error reading the file in " + file_path, exc)
    return read_data


async def getColumnHeaders(df: pandas.DataFrame) -> list:
    columns = list(df.columns)
    return columns



import json


async def asyncReadJSON(file_name):
    read_data = []
    try:
        with open(file_name, "r") as openfile:
            read_data = json.load(openfile)
    except:
        print(f"Error reading JSON file {file_name}")
    return read_data



from .asyncListDir import asyncListDir
from .asyncReadJSON import asyncReadJSON
from .createDirectory import createDirectory
from .getFilePath import getFilePath
from .readData import readData
from .writeFile import writeFile



from pyfiglet import Figlet


def signature(title):
    f = Figlet(font="fender")
    print(f.renderText(title))
    print("*-------------------Author---------------------*")
    print("Ishfaq Ahmed, https://github.com/IshfaqAhmedProg")



import os


async def asyncListDir(input_dir):
    input_files = []
    for file_path in os.listdir(input_dir):
        if os.path.isfile(os.path.join(input_dir, file_path)):
            input_files.append(file_path)
    return input_files
